import java.io.File;

import javafx.application.Application;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.Menu;
import javafx.scene.control.MenuBar;
import javafx.scene.control.MenuItem;
import javafx.scene.control.TextField;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.image.PixelReader;
import javafx.scene.image.PixelWriter;
import javafx.scene.image.WritableImage;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.stage.FileChooser;
import javafx.stage.Screen;
import javafx.stage.Stage;

public class ImageFilter extends Application {

	ImageView imgView;	
	MenuItem openItem;
	MenuItem darkenItem;
	
	public static void main(String[] args) {
		launch();
	}
	
	@Override
	public void start(Stage stage) {
		// Stage title & root node
		stage.setTitle("Image Filter");
		BorderPane root = new BorderPane();

		// Build menus
		MenuBar menuBar = new MenuBar();
		// File menu & items
		Menu fileMenu = new Menu("File");
		openItem = new MenuItem("Open");
		fileMenu.getItems().addAll(openItem);
		// Filter menu & items
		Menu filterMenu = new Menu("Filter");
		darkenItem = new MenuItem("darken");
		filterMenu.getItems().addAll(darkenItem);

		menuBar.getMenus().addAll(fileMenu, filterMenu);

		// EventHandler for menu commands
		MyMenuHandler myMenuHandler = new MyMenuHandler(stage);
		openItem.setOnAction(myMenuHandler);
		darkenItem.setOnAction(myMenuHandler);
		
		Image img = new Image("blocks_main.jpg");
		imgView = new ImageView(img);
		root.setTop(menuBar);
		root.setCenter(imgView);
		Scene scene = new Scene(root);
		stage.setScene(scene);
		stage.sizeToScene();
		stage.show();	
	}
	
	private class MyMenuHandler implements EventHandler<ActionEvent> {
		private Stage stage;

		public MyMenuHandler(Stage stage) {
			this.stage = stage;
		}
		
		@Override
		public void handle(ActionEvent e) {			
			if (e.getSource() == openItem) {
				// Ask user to choose an image file
				FileChooser fileChooser = new FileChooser();
				fileChooser.setInitialDirectory(new File("."));
				fileChooser.setTitle("Open an image");
				File selectedFile = fileChooser.showOpenDialog(stage);
				// Load the chosen image
				if (selectedFile != null) {
					Image img = new Image("file:" + selectedFile.getAbsolutePath());
					imgView.setImage(img);
					double screenWidth = Screen.getPrimary().getVisualBounds().getWidth();
					double screenHeight = Screen.getPrimary().getVisualBounds().getHeight();
					// If image is larger than user's screen resolution, scale the image to fit
					if (img.getWidth() > screenWidth || img.getHeight() > screenHeight) {
						// Scale to fit height to screen
						imgView.setFitHeight(screenHeight);
						imgView.setFitWidth(img.getWidth() * (screenHeight / img.getHeight()));
						// If width is still too large, scale to fit width
						if (imgView.getFitWidth() > screenWidth) {
							imgView.setFitWidth(screenWidth);
							imgView.setFitHeight(img.getHeight() * (screenWidth / img.getWidth()));							
						}
					}
					// Otherwise the image fits, so display it 1:1
					else {
						imgView.setFitHeight(img.getHeight());
						imgView.setFitWidth(img.getWidth());						
					}
					stage.sizeToScene();
				}
			}
			else if (e.getSource() == darkenItem) {
				// Write code here
			}
		}
	}	
}

/** Helper class to process input dialog.
 *  Example usage:
 *		MyInputDialog input = new MyInputDialog();
 *		String answer = input.showInputDialog("Enter a number from 0.0 to 1.0");
 *		if (answer == null) // Quit if user cancels
 *			return;
 *		double num = Double.parseDouble(answer);
 */
class MyInputDialog {
	private String result;
	
	public String showInputDialog(String message) {
		Stage stage = new Stage();
		stage.setTitle("Input Dialog");
		VBox root = new VBox();
		root.setPadding(new Insets(20, 20, 20, 20));
		root.setSpacing(10);
		Label lbl = new Label(message);
		TextField txtField = new TextField();
		HBox hBox = new HBox();
		hBox.setAlignment(Pos.CENTER_RIGHT);
		hBox.setSpacing(10);
		Button okButton = new Button("Ok");
		Button cancelButton = new Button("Cancel");
		okButton.setOnAction(e -> {result = txtField.getText(); stage.close();});
		cancelButton.setOnAction(e -> {result = null; stage.close();});
		hBox.getChildren().addAll(okButton, cancelButton);
		root.getChildren().addAll(lbl, txtField, hBox);
		Scene scene = new Scene(root);
		stage.setScene(scene);
		stage.sizeToScene();
		if (scene.getWidth() < 300) stage.setWidth(300);
		stage.showAndWait();
		return result;
	}
}